@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2011-2013, Cédric Krier
-# Copyright (c) 2013, Nicolas Évrard
-# Copyright (c) 2011-2013, B2CK
+# Copyright (c) 2011-2015, Cédric Krier
+# Copyright (c) 2013-2014, Nicolas Évrard
+# Copyright (c) 2011-2015, B2CK
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,11 +29,12 @@
 
 from __future__ import division
 
-__version__ = '0.4'
-__all__ = ['Flavor', 'Table', 'Literal', 'Column', 'Join', 'Asc', 'Desc']
+__version__ = '0.6'
+__all__ = ['Flavor', 'Table', 'Values', 'Literal', 'Column', 'Join',
+    'Asc', 'Desc']
 
 import string
-from functools import partial
+import warnings
 from threading import local, currentThread
 from collections import defaultdict
 
@@ -115,12 +116,21 @@ class AliasManager(object):
     local = local()
     local.alias = None
     local.nested = 0
+    local.exclude = None
+
+    def __init__(self, exclude=None):
+        if exclude:
+            if getattr(self.local, 'exclude', None) is None:
+                self.local.exclude = []
+            self.local.exclude.extend(exclude)
 
     @classmethod
     def __enter__(cls):
         if getattr(cls.local, 'alias', None) is None:
             cls.local.alias = defaultdict(cls.alias_factory)
             cls.local.nested = 0
+        if getattr(cls.local, 'exclude', None) is None:
+            cls.local.exclude = []
         cls.local.nested += 1
 
     @classmethod
@@ -128,17 +138,20 @@ class AliasManager(object):
         cls.local.nested -= 1
         if not cls.local.nested:
             cls.local.alias = None
+            cls.local.exclude = None
 
     @classmethod
     def get(cls, from_):
         if getattr(cls.local, 'alias', None) is None:
             return ''
-        return cls.local.alias[from_]
+        if from_ in cls.local.exclude:
+            return ''
+        return cls.local.alias[id(from_)]
 
     @classmethod
     def set(cls, from_, alias):
         assert cls.local.alias.get(from_) is None
-        cls.local.alias[from_] = alias
+        cls.local.alias[id(from_)] = alias
 
     @classmethod
     def alias_factory(cls):
@@ -157,6 +170,54 @@ class Query(object):
         yield str(self)
         yield self.params
 
+    def __or__(self, other):
+        return Union(self, other)
+
+    def __and__(self, other):
+        return Intersect(self, other)
+
+    def __sub__(self, other):
+        return Except(self, other)
+
+
+class WithQuery(Query):
+    __slots__ = ('_with',)
+
+    def __init__(self, **kwargs):
+        self._with = None
+        self.with_ = kwargs.pop('with_', None)
+        super(Query, self).__init__(**kwargs)
+
+    @property
+    def with_(self):
+        return self._with
+
+    @with_.setter
+    def with_(self, value):
+        if value is not None:
+            if isinstance(value, With):
+                value = [value]
+            assert all(isinstance(w, With) for w in value)
+        self._with = value
+
+    def _with_str(self):
+        if not self.with_:
+            return ''
+        recursive = (' RECURSIVE' if any(w.recursive for w in self.with_)
+            else '')
+        with_ = ('WITH%s ' % recursive
+            + ', '.join(w.statement() for w in self.with_)
+            + ' ')
+        return with_
+
+    def _with_params(self):
+        if not self.with_:
+            return ()
+        params = []
+        for w in self.with_:
+            params.extend(w.statement_params())
+        return tuple(params)
+
 
 class FromItem(object):
     __slots__ = ()
@@ -166,6 +227,8 @@ class FromItem(object):
         return AliasManager.get(self)
 
     def __getattr__(self, name):
+        if name.startswith('__'):
+            raise AttributeError
         return Column(self, name)
 
     def __add__(self, other):
@@ -179,21 +242,46 @@ class FromItem(object):
         return Join(self, right, type_=type_, condition=condition)
 
 
-class _SelectQueryMixin(object):
-    __slots__ = ('__order_by', '__limit', '__offset')
+class With(FromItem):
+    __slots__ = ('columns', 'query', 'recursive')
+
+    def __init__(self, *columns, **kwargs):
+        self.recursive = kwargs.pop('recursive', False)
+        self.columns = columns
+        self.query = kwargs.pop('query', None)
+        super(With, self).__init__(**kwargs)
+
+    def statement(self):
+        columns = ('(%s)' % ', '.join('"%s"' % c for c in self.columns)
+            if self.columns else '')
+        return '"%s"%s AS (%s)' % (self.alias, columns, self.query)
+
+    def statement_params(self):
+        return self.query.params
+
+    def __str__(self):
+        return '"%s"' % self.alias
+
+    @property
+    def params(self):
+        return tuple()
+
+
+class SelectQuery(WithQuery):
+    __slots__ = ('_order_by', '_limit', '_offset')
 
     def __init__(self, *args, **kwargs):
-        self.__order_by = None
-        self.__limit = None
-        self.__offset = None
+        self._order_by = None
+        self._limit = None
+        self._offset = None
         self.order_by = kwargs.pop('order_by', None)
         self.limit = kwargs.pop('limit', None)
         self.offset = kwargs.pop('offset', None)
-        super(_SelectQueryMixin, self).__init__(*args, **kwargs)
+        super(SelectQuery, self).__init__(*args, **kwargs)
 
     @property
     def order_by(self):
-        return self.__order_by
+        return self._order_by
 
     @order_by.setter
     def order_by(self, value):
@@ -201,7 +289,7 @@ class _SelectQueryMixin(object):
             if isinstance(value, Expression):
                 value = [value]
             assert all(isinstance(col, Expression) for col in value)
-        self.__order_by = value
+        self._order_by = value
 
     @property
     def _order_by_str(self):
@@ -212,13 +300,13 @@ class _SelectQueryMixin(object):
 
     @property
     def limit(self):
-        return self.__limit
+        return self._limit
 
     @limit.setter
     def limit(self, value):
         if value is not None:
             assert isinstance(value, (int, long))
-        self.__limit = value
+        self._limit = value
 
     @property
     def _limit_str(self):
@@ -233,13 +321,13 @@ class _SelectQueryMixin(object):
 
     @property
     def offset(self):
-        return self.__offset
+        return self._offset
 
     @offset.setter
     def offset(self, value):
         if value is not None:
             assert isinstance(value, (int, long))
-        self.__offset = value
+        self._offset = value
 
     @property
     def _offset_str(self):
@@ -249,17 +337,17 @@ class _SelectQueryMixin(object):
         return offset
 
 
-class Select(Query, FromItem, _SelectQueryMixin):
-    __slots__ = ('__columns', '__where', '__group_by', '__having', '__for_',
+class Select(FromItem, SelectQuery):
+    __slots__ = ('_columns', '_where', '_group_by', '_having', '_for_',
         'from_')
 
     def __init__(self, columns, from_=None, where=None, group_by=None,
             having=None, for_=None, **kwargs):
-        self.__columns = None
-        self.__where = None
-        self.__group_by = None
-        self.__having = None
-        self.__for_ = None
+        self._columns = None
+        self._where = None
+        self._group_by = None
+        self._having = None
+        self._for_ = None
         super(Select, self).__init__(**kwargs)
         # TODO ALL|DISTINCT
         self.columns = columns
@@ -271,27 +359,27 @@ class Select(Query, FromItem, _SelectQueryMixin):
 
     @property
     def columns(self):
-        return self.__columns
+        return self._columns
 
     @columns.setter
     def columns(self, value):
         assert all(isinstance(col, Expression) for col in value)
-        self.__columns = tuple(value)
+        self._columns = tuple(value)
 
     @property
     def where(self):
-        return self.__where
+        return self._where
 
     @where.setter
     def where(self, value):
         from sql.operators import And, Or
         if value is not None:
             assert isinstance(value, (Expression, And, Or))
-        self.__where = value
+        self._where = value
 
     @property
     def group_by(self):
-        return self.__group_by
+        return self._group_by
 
     @group_by.setter
     def group_by(self, value):
@@ -299,22 +387,22 @@ class Select(Query, FromItem, _SelectQueryMixin):
             if isinstance(value, Expression):
                 value = [value]
             assert all(isinstance(col, Expression) for col in value)
-        self.__group_by = value
+        self._group_by = value
 
     @property
     def having(self):
-        return self.__having
+        return self._having
 
     @having.setter
     def having(self, value):
         from sql.operators import And, Or
         if value is not None:
             assert isinstance(value, (Expression, And, Or))
-        self.__having = value
+        self._having = value
 
     @property
     def for_(self):
-        return self.__for_
+        return self._for_
 
     @for_.setter
     def for_(self, value):
@@ -322,7 +410,7 @@ class Select(Query, FromItem, _SelectQueryMixin):
             if isinstance(value, For):
                 value = [value]
             assert all(isinstance(f, For) for f in value)
-        self.__for_ = value
+        self._for_ = value
 
     @staticmethod
     def _format_column(column):
@@ -350,13 +438,15 @@ class Select(Query, FromItem, _SelectQueryMixin):
             for_ = ''
             if self.for_ is not None:
                 for_ = ' ' + ' '.join(map(str, self.for_))
-            return ('SELECT %s FROM %s' % (columns, from_) + where + group_by
-                + having + self._order_by_str + self._limit_str +
-                self._offset_str + for_)
+            return (self._with_str()
+                + 'SELECT %s FROM %s' % (columns, from_)
+                + where + group_by + having + self._order_by_str
+                + self._limit_str + self._offset_str + for_)
 
     @property
     def params(self):
         p = []
+        p.extend(self._with_params())
         for column in self.columns:
             if isinstance(column, As):
                 p.extend(column.expression.params)
@@ -374,68 +464,63 @@ class Select(Query, FromItem, _SelectQueryMixin):
                 p.extend(expression.params)
         return tuple(p)
 
-    def __or__(self, other):
-        return Union(self, other)
 
-    def __and__(self, other):
-        return Interesect(self, other)
+class Insert(WithQuery):
+    __slots__ = ('_table', '_columns', '_values', '_returning')
 
-    def __sub__(self, other):
-        return Except(self, other)
-
-
-class Insert(Query):
-    __slots__ = ('__table', '__columns', '__values', '__returning')
-
-    def __init__(self, table, columns=None, values=None, returning=None):
-        self.__table = None
-        self.__columns = None
-        self.__values = None
-        self.__returning = None
+    def __init__(self, table, columns=None, values=None, returning=None,
+            **kwargs):
+        self._table = None
+        self._columns = None
+        self._values = None
+        self._returning = None
         self.table = table
         self.columns = columns
         self.values = values
         self.returning = returning
+        super(Insert, self).__init__(**kwargs)
 
     @property
     def table(self):
-        return self.__table
+        return self._table
 
     @table.setter
     def table(self, value):
         assert isinstance(value, Table)
-        self.__table = value
+        self._table = value
 
     @property
     def columns(self):
-        return self.__columns
+        return self._columns
 
     @columns.setter
     def columns(self, value):
         if value is not None:
             assert all(isinstance(col, Column) for col in value)
             assert all(col.table == self.table for col in value)
-        self.__columns = value
+        self._columns = value
 
     @property
     def values(self):
-        return self.__values
+        return self._values
 
     @values.setter
     def values(self, value):
         if value is not None:
             assert isinstance(value, (list, Select))
-        self.__values = value
+        if isinstance(value, list):
+            value = Values(value)
+        self._values = value
 
     @property
     def returning(self):
-        return self.__returning
+        return self._returning
 
     @returning.setter
     def returning(self, value):
         if value is not None:
             assert isinstance(value, list)
-        self.__returning = value
+        self._returning = value
 
     @staticmethod
     def _format(value, param=None):
@@ -453,32 +538,24 @@ class Insert(Query):
         if self.columns:
             assert all(col.table == self.table for col in self.columns)
             columns = ' (' + ', '.join(map(str, self.columns)) + ')'
-        if isinstance(self.values, list):
-            format_ = partial(self._format, param=Flavor.get().param)
-            values = ' VALUES ' + ', '.join(
-                '(' + ', '.join(map(format_, v)) + ')'
-                for v in self.values)
-            # TODO manage DEFAULT
-        elif isinstance(self.values, Select):
+        if isinstance(self.values, Query):
             values = ' %s' % str(self.values)
+            # TODO manage DEFAULT
         elif self.values is None:
             values = ' DEFAULT VALUES'
         returning = ''
         if self.returning:
             returning = ' RETURNING ' + ', '.join(map(str, self.returning))
-        return 'INSERT INTO %s' % self.table + columns + values + returning
+        with AliasManager():
+            return (self._with_str()
+                + 'INSERT INTO %s' % self.table + columns
+                + values + returning)
 
     @property
     def params(self):
         p = []
-        if isinstance(self.values, list):
-            for values in self.values:
-                for value in values:
-                    if isinstance(value, (Expression, Select)):
-                        p.extend(value.params)
-                    else:
-                        p.append(value)
-        elif isinstance(self.values, Select):
+        p.extend(self._with_params())
+        if isinstance(self.values, Query):
             p.extend(self.values.params)
         if self.returning:
             for exp in self.returning:
@@ -487,37 +564,37 @@ class Insert(Query):
 
 
 class Update(Insert):
-    __slots__ = ('__where', '__values', 'from_')
+    __slots__ = ('_where', '_values', 'from_')
 
     def __init__(self, table, columns, values, from_=None, where=None,
-            returning=None):
+            returning=None, **kwargs):
         super(Update, self).__init__(table, columns=columns, values=values,
-            returning=returning)
-        self.__where = None
+            returning=returning, **kwargs)
+        self._where = None
         self.from_ = From(from_) if from_ else None
         self.where = where
 
     @property
     def values(self):
-        return self.__values
+        return self._values
 
     @values.setter
     def values(self, value):
         if isinstance(value, Select):
             value = [value]
         assert isinstance(value, list)
-        self.__values = value
+        self._values = value
 
     @property
     def where(self):
-        return self.__where
+        return self._where
 
     @where.setter
     def where(self, value):
         from sql.operators import And, Or
         if value is not None:
             assert isinstance(value, (Expression, And, Or))
-        self.__where = value
+        self._where = value
 
     def __str__(self):
         assert all(col.table == self.table for col in self.columns)
@@ -540,12 +617,14 @@ class Update(Insert):
             returning = ''
             if self.returning:
                 returning = ' RETURNING ' + ', '.join(map(str, self.returning))
-            return ('UPDATE %s SET ' % table + values + from_ + where
-                + returning)
+            return (self._with_str()
+                + 'UPDATE %s SET ' % table + values + from_
+                + where + returning)
 
     @property
     def params(self):
         p = []
+        p.extend(self._with_params())
         for value in self.values:
             if isinstance(value, (Expression, Select)):
                 p.extend(value.params)
@@ -561,63 +640,68 @@ class Update(Insert):
         return tuple(p)
 
 
-class Delete(Query):
-    __slots__ = ('__table', '__where', '__returning', 'only')
+class Delete(WithQuery):
+    __slots__ = ('_table', '_where', '_returning', 'only')
 
     def __init__(self, table, only=False, using=None, where=None,
-            returning=None):
-        self.__table = None
-        self.__where = None
-        self.__returning = None
+            returning=None, **kwargs):
+        self._table = None
+        self._where = None
+        self._returning = None
         self.table = table
         self.only = only
         # TODO using (not standard)
         self.where = where
         self.returning = returning
+        super(Delete, self).__init__(**kwargs)
 
     @property
     def table(self):
-        return self.__table
+        return self._table
 
     @table.setter
     def table(self, value):
         assert isinstance(value, Table)
-        self.__table = value
+        self._table = value
 
     @property
     def where(self):
-        return self.__where
+        return self._where
 
     @where.setter
     def where(self, value):
         from sql.operators import And, Or
         if value is not None:
             assert isinstance(value, (Expression, And, Or))
-        self.__where = value
+        self._where = value
 
     @property
     def returning(self):
-        return self.__returning
+        return self._returning
 
     @returning.setter
     def returning(self, value):
         if value is not None:
             assert isinstance(value, list)
-        self.__returning = value
+        self._returning = value
 
     def __str__(self):
-        only = ' ONLY' if self.only else ''
-        where = ''
-        if self.where:
-            where = ' WHERE ' + str(self.where)
-        returning = ''
-        if self.returning:
-            returning = ' RETURNING ' + ', '.join(map(str, self.returning))
-        return 'DELETE FROM%s %s' % (only, self.table) + where + returning
+        with AliasManager(exclude=[self.table]):
+            only = ' ONLY' if self.only else ''
+            where = ''
+            if self.where:
+                where = ' WHERE ' + str(self.where)
+            returning = ''
+            if self.returning:
+                returning = ' RETURNING ' + ', '.join(map(str, self.returning))
+            return (self._with_str()
+                + 'DELETE FROM%s %s' % (only, self.table)
+                + where + returning)
 
     @property
     def params(self):
         p = []
+        p.extend(self._with_params())
         if self.where:
             p.extend(self.where.params)
         if self.returning:
@@ -626,12 +710,12 @@ class Delete(Query):
         return tuple(p)
 
 
-class CombiningQuery(Query, FromItem, _SelectQueryMixin):
+class CombiningQuery(FromItem, SelectQuery):
     __slots__ = ('queries', 'all_')
     _operator = ''
 
     def __init__(self, *queries, **kwargs):
-        assert all(isinstance(q, _SelectQueryMixin) for q in queries)
+        assert all(isinstance(q, Query) for q in queries)
         self.queries = queries
         self.all_ = kwargs.pop('all_', False)
         super(CombiningQuery, self).__init__(**kwargs)
@@ -658,51 +742,68 @@ class Union(CombiningQuery):
     _operator = 'UNION'
 
 
-class Interesect(CombiningQuery):
+class Intersect(CombiningQuery):
     __slots__ = ()
     _operator = 'INTERSECT'
 
 
+class Interesect(Intersect):
+    def __init__(self, *args, **kwargs):
+        warnings.warn('Interesect query is deprecated, use Intersect',
+            DeprecationWarning, stacklevel=2)
+        super(Interesect, self).__init__(*args, **kwargs)
+
+
 class Except(CombiningQuery):
     __slots__ = ()
     _operator = 'EXCEPT'
 
 
 class Table(FromItem):
-    __slots__ = '__name'
+    __slots__ = ('_name', '_schema', '_database')
 
-    def __init__(self, name):
+    def __init__(self, name, schema=None, database=None):
         super(Table, self).__init__()
-        self.__name = name
+        self._name = name
+        self._schema = schema
+        self._database = database
 
     def __str__(self):
-        return '"%s"' % self.__name
+        if self._database:
+            return '"%s"."%s"."%s"' % (
+                self._database, self._schema, self._name)
+        elif self._schema:
+            return '"%s"."%s"' % (self._schema, self._name)
+        else:
+            return '"%s"' % self._name
 
     @property
     def params(self):
         return ()
 
-    def insert(self, columns=None, values=None, returning=None):
+    def insert(self, columns=None, values=None, returning=None, with_=None):
         return Insert(self, columns=columns, values=values,
-            returning=returning)
+            returning=returning, with_=with_)
 
-    def update(self, columns, values, from_=None, where=None, returning=None):
+    def update(self, columns, values, from_=None, where=None, returning=None,
+            with_=None):
         return Update(self, columns=columns, values=values, from_=from_,
-            where=where, returning=returning)
+            where=where, returning=returning, with_=with_)
 
-    def delete(self, only=False, using=None, where=None, returning=None):
+    def delete(self, only=False, using=None, where=None, returning=None,
+            with_=None):
         return Delete(self, only=only, using=using, where=where,
-            returning=returning)
+            returning=returning, with_=with_)
 
 
 class Join(FromItem):
-    __slots__ = ('__left', '__right', '__condition', '__type_')
+    __slots__ = ('_left', '_right', '_condition', '_type_')
 
     def __init__(self, left, right, type_='INNER', condition=None):
         super(Join, self).__init__()
-        self.__left, self.__right = None, None
-        self.__condition = None
-        self.__type_ = None
+        self._left, self._right = None, None
+        self._condition = None
+        self._type_ = None
         self.left = left
         self.right = right
         self.condition = condition
@@ -710,43 +811,43 @@ class Join(FromItem):
 
     @property
     def left(self):
-        return self.__left
+        return self._left
 
     @left.setter
     def left(self, value):
         assert isinstance(value, FromItem)
-        self.__left = value
+        self._left = value
 
     @property
     def right(self):
-        return self.__right
+        return self._right
 
     @right.setter
     def right(self, value):
         assert isinstance(value, FromItem)
-        self.__right = value
+        self._right = value
 
     @property
     def condition(self):
-        return self.__condition
+        return self._condition
 
     @condition.setter
     def condition(self, value):
         from sql.operators import And, Or
         if value is not None:
             assert isinstance(value, (Expression, And, Or))
-        self.__condition = value
+        self._condition = value
 
     @property
     def type_(self):
-        return self.__type_
+        return self._type_
 
     @type_.setter
     def type_(self, value):
         value = value.upper()
         assert value in ('INNER', 'LEFT', 'LEFT OUTER',
             'RIGHT', 'RIGHT OUTER', 'FULL', 'FULL OUTER', 'CROSS')
-        self.__type_ = value
+        self._type_ = value
 
     def __str__(self):
         join = '%s %s JOIN %s' % (From([self.left]), self.type_,
@@ -786,21 +887,22 @@ class From(list):
 
     def __str__(self):
         def format(from_):
+            template = '%s'
             if isinstance(from_, Query):
-                return '(%s) AS "%s"' % (from_, from_.alias)
+                template = '(%s)'
+            alias = getattr(from_, 'alias', None)
+            # TODO column_alias
+            columns_definitions = getattr(from_, 'columns_definitions',
+                None)
+            # XXX find a better test for __getattr__ which returns Column
+            if (alias and columns_definitions
+                    and not isinstance(columns_definitions, Column)):
+                return (template + ' AS "%s" (%s)') % (from_, alias,
+                    columns_definitions)
+            elif alias:
+                return (template + ' AS "%s"') % (from_, alias)
             else:
-                alias = getattr(from_, 'alias', None)
-                columns_definitions = getattr(from_, 'columns_definitions',
-                    None)
-                # XXX find a better test for __getattr__ which returns Column
-                if (alias and columns_definitions
-                        and not isinstance(columns_definitions, Column)):
-                    return '%s AS "%s" (%s)' % (from_, alias,
-                        columns_definitions)
-                elif alias:
-                    return '%s AS "%s"' % (from_, alias)
-                else:
-                    return str(from_)
+                return template % from_
         return ', '.join(map(format, self))
 
     @property
@@ -816,6 +918,35 @@ class From(list):
         return From(super(From, self).__add__([other]))
 
 
+class Values(list, Query, FromItem):
+    __slots__ = ()
+
+    # TODO order, fetch
+
+    def __str__(self):
+        param = Flavor.get().param
+
+        def format_(value):
+            if isinstance(value, Expression):
+                return str(value)
+            else:
+                return param
+        return 'VALUES ' + ', '.join(
+            '(%s)' % ', '.join(map(format_, v))
+            for v in self)
+
+    @property
+    def params(self):
+        p = []
+        for values in self:
+            for value in values:
+                if isinstance(value, Expression):
+                    p.extend(value.params)
+                else:
+                    p.append(value)
+        return tuple(p)
+
+
 class Expression(object):
     __slots__ = ()
 
@@ -943,53 +1074,53 @@ class Expression(object):
 
 
 class Literal(Expression):
-    __slots__ = ('__value')
+    __slots__ = ('_value')
 
     def __init__(self, value):
         super(Literal, self).__init__()
-        self.__value = value
+        self._value = value
 
     @property
     def value(self):
-        return self.__value
+        return self._value
 
     def __str__(self):
         return Flavor.get().param
 
     @property
     def params(self):
-        return (self.__value,)
+        return (self._value,)
 
 Null = None
 
 
 class Column(Expression):
-    __slots__ = ('__from', '__name')
+    __slots__ = ('_from', '_name')
 
     def __init__(self, from_, name):
         super(Column, self).__init__()
-        self.__from = from_
-        self.__name = name
+        self._from = from_
+        self._name = name
 
     @property
     def table(self):
-        return self.__from
+        return self._from
 
     @property
     def name(self):
-        return self.__name
+        return self._name
 
     def __str__(self):
-        if self.__name == '*':
+        if self._name == '*':
             t = '%s'
         else:
             t = '"%s"'
-        alias = self.__from.alias
+        alias = self._from.alias
         if alias:
             t = '"%s".' + t
-            return t % (alias, self.__name)
+            return t % (alias, self._name)
         else:
-            return t % self.__name
+            return t % self._name
 
     @property
     def params(self):
@@ -1063,35 +1194,35 @@ class Desc(Order):
 
 
 class For(object):
-    __slots__ = ('__tables', '__type_', 'nowait')
+    __slots__ = ('_tables', '_type_', 'nowait')
 
     def __init__(self, type_, *tables, **kwargs):
-        self.__tables = None
-        self.__type_ = None
+        self._tables = None
+        self._type_ = None
         self.tables = list(tables)
         self.type_ = type_
         self.nowait = kwargs.get('nowait')
 
     @property
     def tables(self):
-        return self.__tables
+        return self._tables
 
     @tables.setter
     def tables(self, value):
         if not isinstance(value, list):
             value = [value]
         all(isinstance(table, Table) for table in value)
-        self.__tables = value
+        self._tables = value
 
     @property
     def type_(self):
-        return self.__type_
+        return self._type_
 
     @type_.setter
     def type_(self, value):
         value = value.upper()
         assert value in ('UPDATE', 'SHARE')
-        self.__type_ = value
+        self._type_ = value
 
     def __str__(self):
         tables = ''
