@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright (c) 2011-2013, Cédric Krier
-# Copyright (c) 2013, Nicolas Évrard
+# Copyright (c) 2013-2014, Nicolas Évrard
 # Copyright (c) 2011-2013, B2CK
 # All rights reserved.
 #
@@ -28,8 +28,10 @@
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import unittest
+import warnings
+from copy import deepcopy
 
-from sql import Table, Join, Union, Literal, Flavor, For
+from sql import Table, Join, Union, Literal, Flavor, For, With
 from sql.functions import Now, Function
 from sql.aggregate import Min
 
@@ -107,6 +109,16 @@ class TestSelect(unittest.TestCase):
         self.assertEqual(str(intersect),
             'SELECT * FROM "t" AS "a" INTERSECT SELECT * FROM "t2" AS "b"')
 
+        from sql import Intersect, Interesect
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            interesect = Interesect(query1, query2)
+            self.assertEqual(len(w), 1)
+            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
+            self.assertIn('Interesect query is deprecated, use Intersect',
+                str(w[-1].message))
+        self.assertTrue(isinstance(interesect, Intersect))
+
     def test_select_except(self):
         query1 = self.table.select()
         query2 = Table('t2').select()
@@ -224,3 +236,19 @@ class TestSelect(unittest.TestCase):
         self.assertEqual(str(query),
             'SELECT "a"."c" FROM "t" AS "a" FOR UPDATE')
         self.assertEqual(query.params, ())
+
+    def test_copy(self):
+        query = self.table.select()
+        copy_query = deepcopy(query)
+        self.assertNotEqual(query, copy_query)
+        self.assertEqual(str(copy_query), 'SELECT * FROM "t" AS "a"')
+        self.assertEqual(copy_query.params, ())
+
+    def test_with(self):
+        w = With(query=self.table.select(self.table.c1))
+
+        query = w.select(with_=[w])
+        self.assertEqual(str(query),
+            'WITH "a" AS (SELECT "b"."c1" FROM "t" AS "b") '
+            'SELECT * FROM "a" AS "a"')
+        self.assertEqual(query.params, ())
